'------------------------------------------------------------------------------
'     (c) 2004 BiPOM Electronics
'------------------------------------------------------------------------------
' This module is presented here only to serve as a sample for
' 8051 Basic program development on the MINI-MAX/51-C system.
' BiPOM Electronics does not guarantee its functionality
' with other micro-controller systems.
'
' Please e-mail any questions and suggestions to info@bipom.com
'
' Module: ENCODER.BAS
'
' Revision: 1.02
'
' Description: This module converts pulses of digital rotary encoder to linear position 
'
'-------------------------------------------------------------------------------
' MINI-MAX/51-C Configuration
$BAUD = 19200  'set 19200 baud rate for
$CRYSTAL = 22118400 '22.1184 MHz crystal

'-------- HARDWARE DESCRIPTION -------------------------------------------------

Phase_A Alias P3.2  ' phase A on the encoder connected with P3.2 (EX0 )
Phase_B Alias P3.3  ' phase B on the encoder connected with P3.6 
PB_Zero Alias P1.5  'Push  Button <ZERO> - Reset counter (when low level on P1.5)


'--------- CONSTANTS ------------------------------------------------------------
										'With Direction variable You can to select positive and negative
										' direction on motion 	
Const Direction = 1 ' Direction=1 - increment counter  when have negative edge(1->0)
										'               of phase A and phase B is in high  level 
										' Direction=0 - increment counter when have negative edge (1->0)
										'               of phase A and phase B is in low  level
										' NOTE: Direction  const  can have value  0 or 1 only !
										
Const Pulses_mm = 20' pulses from encoder for 1 mm linear motion

Const Coeff_mm  = 100/Pulses_mm ' Coefficient for conversion pulses to mm

'--------- VARIABLES -------------------------------------------------------------

Dim Count As Integer     ' range of distance from -327.65 to +327.65 mm
Dim mm_whole As Integer  ' whole part of distance
Dim mm_frac  As Integer  ' fractional part of distance
Dim temp   As Integer    ' temp variable for calculating result
Dim Sign   As Byte       ' Sign of Distance
Dim FlgChg As Byte       ' Flag for changed distance value

'--------- INITIALIZING ----------------------------------------------------------

On Int0 Int0_int  'Initialise the INT0 Interrupt
' On Int0 Int0_int Nosave     
' Use The Nosave Option When You Dont Want The Internal Registers To Be Saved/restored In
' The Interrupt Routine
Enable Interrupts
Enable Int0        'enable the interrupt
Priority Set Int0  'set INT0 to the highest priority level (INT0 has by default the highest prio)
Set TCON.0 'IT0=1 -external interrupt 0 : edge - triggered  


' Clear the Read/Write line wo it is always Write to the LCD
Reset P0.1

call init_lcd

'4-line interface to the LCD
Config Lcdbus = 4		

'Configure the port lines for LCD
Config Lcdpin = Pin , Db4 = P0.4 , Db5 = P0.5 , Db6 = P0.6 , Db7 = P0.7 , E = P0.2 , Rs = P0.0

'Configure lcd screen; 'other options are 16 * 4 and 20 * 4, 20 * 2 , 16 * 1a
'When you dont include this option 16 * 2 is assumed
'16 * 1a is intended for 16 character displays with split addresses over 2 lines
Config Lcd = 24 * 2 

'Configure I2C Bus for birghtness control
Config Sda = P1.7
Config Scl = P1.6
Config I2cdelay = 2

'Setup brightness for the LCD
I2csend &HB0 , 0
Reset P0.1
Reset P1.5
I2csend &HB0 , 0
I2cstart			'start condition
I2cwbyte &HB0       'slave address
I2cwbyte 2          'value to write ( 0: brightest, 15: dimmest )
I2cstop             'stop condition
Set P1.5

Cls  'clear the LCD display


Count=0  ' Clearing the Count variable
FlgChg=1 ' Having change on Count variable


LCD "linear motion"
wait 2 ' delay time 2 sec



'--------------------- Loop of the Main Program ---------------------------------
while 1 = 1 
	If FlgChg = 1 Then  ' If have change Count-variable then  have display refreshing 
		FlgChg =0         ' Clear flag for change Count variable 
		if Count < 0 Then ' If Distance have negative sign
			Sign=1          ' sing on the distanec is minus
			temp= 0 - Count ' Calculate abs value on the distance 
		Else              ' If Distance have positive sign or zero value
			Sign=0          ' sign  on the distance is plus 
			temp=Count      ' Calculate abs value on the distance 
		End if				
		mm_whole = temp / 100 ' Calculating whole part of distance
		mm_frac  = mm_whole *100   
		mm_frac  = temp - mm_frac'Calculating fractional  part of distance
		
		Cls  'clear the LCD display
		if Sign =1 Then 
			if mm_frac < 10 Then 
				LCD "Distance = -";mm_whole;".0";mm_frac;" mm"
			Else 
				LCD "Distance = -";mm_whole;".";mm_frac ;" mm"
			End if
		Else
			if mm_frac < 10 Then 
				LCD "Distance = +";mm_whole;".0";mm_frac;" mm"
			Else 
				LCD "Distance = +";mm_whole;".";mm_frac ;" mm"
			End if		
		End if	
	End If
	
	if PB_Zero = 0 Then ' If the Button <ZERO> is  pressed then Clearing the Count variable
		FlgChg = 1        ' Having change on Count variable
		Count = 0         ' Clearing the Count variable
	End if	
wend

'-------------- The Interrupt Handler For The Int0 Interrupt ----------------------
Int0_int:
	FlgChg =1
	If Phase_B = Direction Then
		Count = Count + Coeff_mm
	Else		
		Count = Count - Coeff_mm
	End If
Return  